@@ -174,6 +174,7 @@ set_reply (DBusPendingCall *pending, void *user_data)
     void **replyptr = (void **)user_data;
 
     *replyptr = dbus_pending_call_steal_reply (pending);
+  dbus_pending_call_unref (pending);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -227,10 +228,7 @@ get_registered_event_listeners (SpiBridge *app)
     }
   dbus_message_iter_init (reply, &iter);
   dbus_message_iter_recurse (&iter, &iter_array);
-  /* TODO: This is bad. Need to determine that the array is non-empty,
-     so that we don't initially read a value rom it in that case, but using
-     a deprecated function. */
-  if (dbus_message_iter_get_array_len (&iter_array) > 0) do
+  while (dbus_message_iter_get_arg_type (&iter_array) != DBUS_TYPE_INVALID)
     {
       char *bus_name, *event;
       dbus_message_iter_recurse (&iter_array, &iter_struct);
@@ -238,8 +236,8 @@ get_registered_event_listeners (SpiBridge *app)
       dbus_message_iter_next (&iter_struct);
       dbus_message_iter_get_basic (&iter_struct, &event);
       add_event (bus_name, event);
+      dbus_message_iter_next (&iter_array);
     }
-  while (dbus_message_iter_next (&iter_array));
   dbus_message_unref (reply);
 }
 
@@ -250,7 +248,8 @@ register_reply (DBusPendingCall *pending, void *user_data)
   SpiBridge *app = user_data;
   DBusMessage *message;
 
-    reply = dbus_pending_call_steal_reply (pending);
+  reply = dbus_pending_call_steal_reply (pending);
+  dbus_pending_call_unref (pending);
   if (reply)
     {
       gchar *app_name, *obj_path;
@@ -306,7 +305,8 @@ register_application (SpiBridge * app)
   dbus_message_iter_init_append (message, &iter);
   spi_object_append_reference (&iter, app->root);
   
-    if (!dbus_connection_send_with_reply (app->bus, message, &pending, -1))
+    if (!dbus_connection_send_with_reply (app->bus, message, &pending, -1)
+        || !pending)
     {
         return FALSE;
     }
@@ -323,6 +323,8 @@ app->app_bus_addr = g_malloc(max_addr_length * sizeof(char));
 #ifndef DISABLE_P2P
 sprintf(app->app_bus_addr, "unix:path=/tmp/at-spi2/socket-%d-%d", getpid(),
 rand());
+#else
+  app->app_bus_addr [0] = '\0';
 #endif
 
   return TRUE;
